from abstract_gui import AbstractWindowManager,make_component
from abstract_webtools import UserAgentManager,UrlManager,SafeRequest,SoupManager,LinkManager,CipherManager

class GuiGrabber:
    def __init__(self,url="www.example.com"):
        self.window_mgr = AbstractWindowManager()
        self.window_name = self.window_mgr.add_window(title="Gui_Grabber",layout=[],event_handlers=[self.while_window])
        self.url = url
        self.parse_type_choices = ['html.parser', 'lxml', 'html5lib']
        self.window_mgr.while_window()
    def layout(event,values,window):
        # Add a dropdown for selecting BeautifulSoup parsing capabilities
        make_component("theme",'LightGrey1')
        layout = [[make_component("Text",'URL:', size=(8, 1)),
                   make_component("Input",url, key='-URL-',enable_events=True),
                   make_component("Text",'status:'),
                   make_component("Text",'200',key="-STATUS_CODE-"),
                   make_component("Text",f'success: {self.url} is valid',key="-URL_WARNING-"),
                   make_component("Button",'Grab URL',key='-GRAB_URL-',visible=True)],
            [make_component("Checkbox",'Custom User-Agent', default=False, key='-CUSTOMUA-', enable_events=True)],
            [make_component("Text",'User-Agent:', size=(8, 1)),
             make_component("Combo",get_user_agents(), default_value='Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/89.0.4389.90 Safari/537.36', key='-USERAGENT-', disabled=False)],
            [self.get_cypher_checks()],
            [make_component("Button",'Grab URL'),
             make_component("Button",'Action'),
             make_component("Button",'Get All Text')],
            [make_component("Text",'Parsing Capabilities:', size=(15, 1)),
             make_component("DropDown",parse_type_choices, default_value='html.parser', key='-parse_type-',enable_events=True)],
            [get_multi_line({"key":'-SOURCECODE-'})],
            [make_component("Text",'find soup:'),[[
                make_component("Checkbox",'',default=True,key='-SOUP_TAG_BOOL-',enable_events=True),
                make_component("Combo",[], size=(15, 1),key='-SOUP_TAG-',enable_events=True)],
                                    [make_component("Checkbox",'',default=False,key='-SOUP_ATTRIBUTE_BOOL-',enable_events=True),
                                     make_component("Combo",[], size=(15, 1),key='-SOUP_ATTRIBUTE-',enable_events=True)],
                                    [make_component("Checkbox",'',default=False,key='-SOUP_ATTRIBUTE_1_BOOL-',enable_events=True),
                                     make_component("Combo",[], size=(15, 1),key='-SOUP_ATTRIBUTE_1-',enable_events=True)],
                                    [make_component("Checkbox",'',default=False,key='-SOUP_ATTRIBUTE_2_BOOL-',enable_events=True),
                                     make_component("Combo",[], size=(15, 1),key='-SOUP_ATTRIBUTE_2-',enable_events=True)],
                                    make_component("Input",key='-SOUP_VALUES_INPUT-'),
                                                  make_component("Button",'get soup'),
                                                  make_component("Button",'all soup'),
                                                  make_component("Button",'Send Soup')]],
                  [get_multi_line({"key":"-FIND_ALL_OUTPUT-"})]]
        return layout
GuiGrabber()



from ..abstract_webtools import *
import random
operating_systems = ['Macintosh','Windows','Linux']
browsers = ['Firefox','Chrome','IceDragon','Waterfox','Gecko','Safari','MetaSr']
def get_itter(iter_input,itter_list):
    if not iter_input:
        return itter_list[0]
    if iter_input in itter_list:
        return iter_input
    iter_input_lower = iter_input.lower()
    for itter in itter_list:
        itter_lower = itter.lower()
        if iter_input_lower in itter_lower:
            return itter
    return itter_list[0]
def get_browser(browser=None):
    return get_itter(browser,browsers)
def get_operating_system(operating_system=None):
    return get_itter(operating_system,operating_systems)
class UserAgentManager:
    def __init__(self, operating_system=None, browser=None, version=None,user_agent=None):
        self.operating_system = get_operating_system(operating_system=operating_system)
        self.browser = get_browser(browser=browser)
        self.version = version or '42.0'
        self.user_agent = user_agent or self.get_user_agent()
        self.header = self.user_agent_header()
    @staticmethod
    def user_agent_db():
        from ..big_user_agent_list import big_user_agent_dict
        return big_user_agent_dict

    def get_user_agent(self):
        ua_db = self.user_agent_db()

        if self.operating_system and self.operating_system in ua_db:
            operating_system_db = ua_db[self.operating_system]
        else:
            operating_system_db = random.choice(list(ua_db.values()))

        if self.browser and self.browser in operating_system_db:
            browser_db = operating_system_db[self.browser]
        else:
            browser_db = random.choice(list(operating_system_db.values()))

        if self.version and self.version in browser_db:
            return browser_db[self.version]
        else:
            return random.choice(list(browser_db.values()))

    def user_agent_header(self):
        return {"user-agent": self.user_agent}
class UserAgentManagerSingleton:
    _instance = None
    @staticmethod
    def get_instance(user_agent=UserAgentManager().get_user_agent()[0]):
        if UserAgentManagerSingleton._instance is None:
            UserAgentManagerSingleton._instance = UserAgentManager(user_agent=user_agent)
        elif UserAgentManagerSingleton._instance.user_agent != user_agent:
            UserAgentManagerSingleton._instance = UserAgentManager(user_agent=user_agent)
        return UserAgentManagerSingleton._instance



from ..abstract_webtools import *
class TLSAdapter(HTTPAdapter):
    def __init__(self, ssl_manager=None,ciphers=None, certification: Optional[str] = None, ssl_options: Optional[List[str]] = None):
        if ssl_manager == None:
            ssl_manager = SSLManager(ciphers=ciphers, ssl_options=ssl_options, certification=certification)
        self.ssl_manager = ssl_manager
        self.ciphers = ssl_manager.ciphers
        self.certification = ssl_manager.certification
        self.ssl_options = ssl_manager.ssl_options
        self.ssl_context = self.ssl_manager.ssl_context
        super().__init__()

    def init_poolmanager(self, *args, **kwargs):
        kwargs['ssl_context'] = self.ssl_context
        return super().init_poolmanager(*args, **kwargs)
class TLSAdapterSingleton:
    _instance: Optional[TLSAdapter] = None
    
    @staticmethod
    def get_instance(ciphers: Optional[List[str]] = None, certification: Optional[str] = None, ssl_options: Optional[List[str]] = None) -> TLSAdapter:
        if (not TLSAdapterSingleton._instance) or (
            TLSAdapterSingleton._instance.ciphers != ciphers or 
            TLSAdapterSingleton._instance.certification != certification or 
            TLSAdapterSingleton._instance.ssl_options != ssl_options
        ):
            TLSAdapterSingleton._instance = TLSAdapter(ciphers=ciphers, certification=certification, ssl_options=ssl_options)
        return TLSAdapterSingleton._instance



class CipherManager:
    @staticmethod
    def  get_default_ciphers()-> list:
        return [
            "ECDHE-RSA-AES256-GCM-SHA384", "ECDHE-ECDSA-AES256-GCM-SHA384",
            "ECDHE-RSA-AES256-SHA384", "ECDHE-ECDSA-AES256-SHA384",
            "ECDHE-RSA-AES256-SHA", "ECDHE-ECDSA-AES256-SHA",
            "ECDHE-RSA-AES128-GCM-SHA256", "ECDHE-RSA-AES128-SHA256",
            "ECDHE-ECDSA-AES128-GCM-SHA256", "ECDHE-ECDSA-AES128-SHA256",
            "AES256-SHA", "AES128-SHA"
        ]

    def __init__(self,cipher_list=None):
        if cipher_list == None:
            cipher_list=self.get_default_ciphers()
        self.cipher_list = cipher_list
        self.create_list()
        self.ciphers_string = self.add_string_list()
    def add_string_list(self):
        if len(self.cipher_list)==0:
            return ''
        return','.join(self.cipher_list)
    def create_list(self):
        if self.cipher_list == None:
            self.cipher_list= []
        elif isinstance(self.cipher_list, str):
            self.cipher_list=self.cipher_list.split(',')
        if isinstance(self.cipher_list, str):
            self.cipher_list=[self.cipher_list]
class CipherManagerSingleton:
    _instance = None
    @staticmethod
    def get_instance(cipher_list=None):
        if CipherManagerSingleton._instance is None:
            CipherManagerSingleton._instance = CipherManager(cipher_list=cipher_list)
        elif CipherManagerSingleton._instance.cipher_list != cipher_list:
            CipherManagerSingleton._instance = CipherManager(cipher_list=cipher_list)
        return CipherManagerSingleton._instance
